<?php

namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\Remote\Exception as RemoteException;

class Tco01100DocumentoPropostaService extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $tcoUtil;
    protected $tcoList;
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $fileUploader;
    protected $fileServerURL;
    protected $entityManager;
    protected $dataSourceName;
    protected $dataSourceManager;

    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $newCode, $tcoUtil, $tcoList, $fileUploader, $tco01100GeneralLibPropCom, $fileServerURL){
        $this->tcoUtil                   = $tcoUtil;
        $this->newCode                   = $newCode;
        $this->tcoList                   = $tcoList;
        $this->connection                = $entityManager->getConnection();
        $this->crudService               = $crudService;
        $this->environment               = $environment;
        $this->fileUploader              = $fileUploader;
        $this->entityManager             = $entityManager;
        $this->fileServerURL             = $fileServerURL;
        $this->dataSourceManager         = $dataSourceManager;
        $this->tco01100GeneralLibPropCom = $tco01100GeneralLibPropCom;

        if(substr($this->fileServerURL, -1)!='/'){
            $this->fileServerURL .= '/';
        }
    }

    /**
     * Faz upload do arquivo para o fileServer do 
     * sistema.
     * 
     * @param {DTO\Request} $request Requisição vinda
     *  do usuário.
     * @param {DTO\Reponse} $response Instância de 
     *  resposta a requisição do usuário.
     */
    public function uploadFile($request, $response){
        $row = $request->getRow();
        $nrOrg = $this->environment->getNrOrg();
        $nrDocumentoPropCom = $this->newCode->getCodeByTableName('DOCUMENTOPROPCOM', $nrOrg, 1, 10, true);
        
        try{
            $this->fileUploader->uploadFilesInRow($row, 'FILE', 'Tco/Documentos');
            $file = $row->get('FILE')[0];

        } catch(RemoteException $e){
            $response->setError(new Error($e->getMessage(), 1));
        }

        $row->set('NRDOCUMENTOPROPCOM', $nrDocumentoPropCom);
        $row->set('DSCAMINHOFILESERVER', $file['path']);
        $row->set('NMDOCUMENTOPROPCOM', $file['name']);
        $row->set('__is_new', true);

        $reqRPath   = $request->getRoutePath();
        $reqMethod  = $request->getMethod();
        $reqUserId  = $request->getUserId();

        $request   = new DTO\Request\Row($row, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($request, 'documentopropcom');
        $dataSet   = new DataSet('documentopropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);

    }

    /**
     * Remove arquivo do fileServer do sistema e
     * o registro do banco de dados.
     * 
     * @param {Array[Int]} $filesToRemove Códigos
     *  de documentos da proposta a serem removidos.
     */
    public function removeFile($filesToRemove){
        $pathToRemove = $this->getDocPropComDetails($filesToRemove);
        $this->removeFileRegister($filesToRemove);

        foreach ($pathToRemove as $registerPath) {
            $this->fileUploader->deleteFile($registerPath['DSCAMINHOFILESERVER']);
        }
    }

    /**
     * Faz download do arquivo para o cliente.
     * 
     * @param {Int} $nrDocumentoPropCom Numero do
     *  documento a ser baixado
     * 
     * @return {BinaryFile|Boolean} Retorna o 
     *  arquivo baixado ou falso caso não encontre.
     */
    public function downloadFile($nrDocumentoPropCom){
        $file = $this->getDocPropComDetails(array($nrDocumentoPropCom));
        $downloadUrl = $this->fileServerURL;

        if(isset($file[0])){
            $fileName = $file[0]['NMDOCUMENTOPROPCOM'];
            $downloadUrl .= $file[0]['DSCAMINHOFILESERVER'];
            $file = base64_encode(file_get_contents($downloadUrl));
            return array(
                "FILE_NAME"=>$fileName,
                "FILE_B64"=>$file
            );
        } else return false;
    }

    /**
     * Remove o registro do arquivo no banco de dados.
     * 
     * @param {Array[Int]} $filesToRemove Códigos
     *  de documentos da proposta a serem removidos.
     */
    private function removeFileRegister($filesToRemove){
        $sql = "DELETE FROM 
                    DOCUMENTOPROPCOM 
                WHERE 
                    VALOR_NA_LISTA(NRDOCUMENTOPROPCOM, :NRDOCUMENTOPROPCOM) > 0";

        $this->tco01100GeneralLibPropCom->setListOnParam($filesToRemove);

        $params = array(
            "NRDOCUMENTOPROPCOM" => $filesToRemove
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);

        $this->tcoList->apagarListaValores($filesToRemove);
    }

    /**
     * Retornar os detalhes dos arquivos referentes no
     * servidor de arquivos.
     * 
     * @param {Array[Int]} $filesToRemove Códigos
     *  de documentos da proposta a serem removidos.
     */
    public function getDocPropComDetails($filesToGet){
        $sql = "SELECT
                    NRDOCUMENTOPROPCOM,
                    NMDOCUMENTOPROPCOM,
                    NRPROPOSTACOMERCIAL,
                    DSCAMINHOFILESERVER 
                FROM 
                    DOCUMENTOPROPCOM 
                WHERE 
                    VALOR_NA_LISTA(NRDOCUMENTOPROPCOM, :NRDOCUMENTOPROPCOM) > 0";

        $this->tco01100GeneralLibPropCom->setListOnParam($filesToGet);

        $params = array(
            "NRDOCUMENTOPROPCOM" => $filesToGet
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        $this->tcoList->apagarListaValores($filesToGet);
        return $result;
    }

     /**
     * Valida se o operador passado por parâmetro é um 
     * responsável válido, ou respectivo lider, pela 
     * proposta comercial.
     * 
     * @param {String} $cdOperador Código do Operador.
     * @param {String} $nrPropostaComercial Número da proposta
     *  comercial.
     */
    public function isValidResponsavelPropCom($cdOperador, $nrPropostaComercial){
        return $this->tco01100GeneralLibPropCom->isValidResponsavelPropCom($cdOperador, $nrPropostaComercial);
    }
}